<?php

/**
* @package FloydWarshall
* @author Janne Mikkonen <janne dot mikkonen at julmajanne dot com>
* @date $Date: 2005/09/10 05:10:48 $
* @version $Revision: 1.0 $
* @license GNU GPL
*/

//error_reporting(E_ALL);
define('INFINITE', pow(2, (20 * 8 - 2)-1));

class FloydWarshall {

    /**
    * Distances array
    * @var array
    */
    var $dist = array(array());
    /**
    * Predecessor array
    * @var array
    */
    var $pred = array(array());
    /**
    * Weights array
    * @var array
    */
    var $weights;
    /**
    * Number of nodes
    * @var integer
    */
    var $nodes;
    /**
    * Node names array
    * @var array
    */
    var $nodenames;
    /**
    * Temporary table for various stuff.
    * @var array
    */
    var $tmp = array();

    /**
    * Constructor
    * @param array $graph Graph matrice.
    * @param array $nodenames Node names as an array.
    */
    function FloydWarshall ($graph, $nodenames='') {

        $this->weights = $graph;
        $this->nodes   = count($this->weights);
        if ( ! empty($nodenames) && $this->nodes == count($nodenames) ) {
            $this->nodenames = $nodenames;
        }
        $this->__floydwarshall();

    }

    /**
    * Constructor for PHP5
    * @param array $graph Graph matrice.
    * @param array $nodenames Node names as an array.
    */
    function __construct($graph, $nodenames='') {
        $this->FloydWarshall($graph, $nodenames);
    }

    /**
    * The actual PHP implementation of Floyd-Warshall algorithm.
    */
    function __floydwarshall () {

        // Initialization
        for ( $i = 0; $i < $this->nodes; $i++ ) {
            for ( $j = 0; $j < $this->nodes; $j++ ) {
                if ( $i == $j ) {
                    $this->dist[$i][$j] = 0;
                } else if ( $this->weights[$i][$j] > 0 ) {
                    $this->dist[$i][$j] = $this->weights[$i][$j];
                } else {
                    $this->dist[$i][$j] = INFINITE;
                }
                $this->pred[$i][$j] = $i;
            }
        }

        // Algorithm

        for ( $k = 0; $k < $this->nodes; $k++ ) {
            for ( $i = 0; $i < $this->nodes; $i++ ) {
                for ( $j = 0; $j < $this->nodes; $j++ ) {
                    if ($this->dist[$i][$j] > ($this->dist[$i][$k] + $this->dist[$k][$j])) {
                        $this->dist[$i][$j] = $this->dist[$i][$k] + $this->dist[$k][$j];
                        $this->pred[$i][$j] = $this->pred[$k][$j];
                    }
                }
            }
        }

    }

    /**
    * Helper function for get_path.
    */
    function __get_path($i, $j) {
		$this->tmp=Array();
        if ( $i != $j ) {
            $this->__get_path($i, $this->pred[$i][$j]);
        }
        array_push($this->tmp, $j);
    }

    /**
    * Get nodes between a and b.
    * @param ingeger $i Starting node.
    * @param integer $j End node.
    * @return array Return array of nodes.
    */
    function get_path($i, $j) {
        $this->__get_path($i, $j);
        return $this->tmp;
    }

    /**
    * Print nodes from a and b.
    * @param ingeger $i Starting node.
    * @param integer $j End node.
    * @return void
    */
    function print_path($i, $j) {

        if ( $i != $j ) {
            $this->print_path($i, $this->pred[$i][$j]);
        }

        if (! empty($this->nodenames) ) {
            print($this->nodenames[$j]) . ' ';
        } else {
            print($j) . ' ';
        }

    }

    /**
    * Get total cost (distance) between point a to b.
    */
    function get_distance($i, $j) {
        return $this->dist[$i][$j];
    }

    /************************************************************
    ***                    DEBUG FUNCTIONS                    ***
    ***                    - print_graph                      ***
    ***                    - print_dist                       ***
    ***                    - print_pred                       ***
    *************************************************************/

    /**
    * Print out the original Graph matrice.
    */
    function print_graph () {
        $rt= '<table id="table_graph" class="managetable" border="1" cellpadding="4">';
        if (! empty($this->nodenames) ) {
            $rt.= '<tr>';
            $rt.= '<td>&nbsp;</td>';
            for ($n = 0; $n < $this->nodes; $n++) {
                $rt.= '<td align="center">' .
                     $this->nodenames[$n] .
                     '</td>';
            }
        }
        $rt.= '</tr>';
        for ($i = 0; $i < $this->nodes; $i++) {
            $rt.= '<tr>';
            if (! empty($this->nodenames) ) {$rt.= '<td align="center">'.$this->nodenames[$i] .'</td>';}
            for ($j = 0; $j < $this->nodes; $j++) {
                $rt.= '<td>' .
                     $this->weights[$i][$j] . '</td>';
            }
            $rt.= '</tr>';
        }
        $rt.= '</table><br />';
	return $rt;
    }

    /**
    * Print out distances matrice.
    */
    function print_dist () {
        $rt= '<table id="table_dist" class="managetable" border="1" style="font-size:9px;" >'."\n";
        if (! empty($this->nodenames) ) {
            $rt.= '<tr>';
            $rt.= '<td>&nbsp;</td>'."\n";
            for ($n = 0; $n < $this->nodes; $n++) {$rt.= '<td align="center">'.$this->nodenames[$n].'</td>';}
        }
        $rt.= '</tr>';
        for ($i = 0; $i < $this->nodes; $i++) {
            $rt.= '<tr style="text-align:center">';
            if (! empty($this->nodenames) ) {$rt.= '<td>'.$this->nodenames[$i].'</td>'."\n";}
            for ($j = 0; $j < $this->nodes; $j++) {$rt.= '<td>'.$this->dist[$i][$j] .'</td>'."\n";}
            $rt.= '</tr>';
        }
        $rt.= '</table><br />'."\n";
	return $rt;
    }

    /**
    * Print out predecessors matrice.
    */
    function print_pred () {
        $rt= '<table id="table_pred" class="managetable" border="1" style="font-size:9px;">'."\n";
        if (! empty($this->nodenames) ) {
            $rt.= '<tr>';
            $rt.= '<td>&nbsp;</td>';
            for ($n = 0; $n < $this->nodes; $n++) {$rt.= '<td>' .$this->nodenames[$n] .'</td>'."\n";}
        }
        $rt.= '</tr>';
        for ($i = 0; $i < $this->nodes; $i++) {
            $rt.= '<tr>';
            if (! empty($this->nodenames) ) {$rt.= '<td>'.$this->nodenames[$i].'['.$i.']</td>'."\n";}
            for ($j = 0; $j < $this->nodes; $j++) {
                $rt.= '<td align="center">'.$this->pred[$i][$j] . '</td>'."\n";
            }
            $rt.= '</tr>'."\n";
        }
        $rt.= '</table><br />'."\n";
	return $rt;
    }

} // End of class
?>